grammar Cminus;

options {
    output = AST;
}

tokens {
    VARDEF;
    ASSIGN;
    COMPARE;
    UPLUS;
    UMINUS;
    BLOCK;
}

program
    : statements EOF!
    ;
    
statements
    : (statement)+
    ;

statement
    : declaration DELIM!
    | assingment DELIM!
    | print DELIM!
    | input DELIM!
    | ifnt
    | whilent
    ;

declaration
    : VAR ID (COMMA ID)* -> ^(VARDEF ID)+
    ;

assingment
    : ID EQ expr -> ^(ASSIGN ID expr)
    ;

print
    : PRINT^ expr
    ;

input
    : INPUT^ ID
    ;

ifnt
    : IF^ LPAR! condition RPAR! block (ELSE! block)?
    ;

whilent
    : WHILE^ LPAR! condition RPAR! block
    ;

block
    : LBR statements RBR -> ^(BLOCK statements)
    ;

condition
    : expr (LT | EQ | GT)^ expr
    ;

expr
    : t1=term (PLUS^ t2=term | MINUS^ t2=term)*
    ;

term
    : f1=factor (MULT^ f2=factor | DIV^ f2=factor)*
    ;
    
factor
    : PLUS factor -> ^(UPLUS factor)
    | MINUS factor -> ^(UMINUS factor)
    | power
    ;

power
    : atom (POW^ f=factor)?
    ;
    
atom
    : ID
    | DOUBLE
    | PI
    | E
    | LPAR expr RPAR -> ^(expr)
    ;

VAR
    : 'var'
    ;

IF
    : 'if'
    ;

ELSE
    : 'else'
    ;

PRINT
    : 'print'
    ;

INPUT
    : 'input'
    ;

WHILE
    : 'while'
    ;

PI
    : 'PI'
    ;

E
    : 'E'
    ;

ID
    : LETTER (LETTER | INT)*
    ;

fragment LETTER
    : 'a'..'z'
    | 'A'..'Z'
    ;

DOUBLE
    : INT
    | '.' INT
    | INT '.'
    | INT '.' INT
    ;

fragment INT
    : '0'..'9'+
    ;

PLUS
    : '+'
    ;
    
MINUS
    : '-'
    ;
    
MULT
    : '*'
    ;
    
DIV
    : '/'
    ;
    
POW
    : '**'
    ;

LPAR
    : '('
    ;

RPAR
    : ')'
    ;

LBR
    : '{'
    ;

RBR
    : '}'
    ;

DELIM
    : ';'
    ;

COMMA
    : ','
    ;

EQ
    : '='
    ;

LT
    : '<'
    ;

GT
    : '>'
    ;

WHITESPACE
    : ('\t'|' '|'\r'|'\n'|'\u000C')+ {$channel = HIDDEN;}
    ;

COMMENT
    : '//' ~('\n'|'\r')* '\r'? '\n' {$channel=HIDDEN;}
    | '/*' ( options {greedy=false;} : . )* '*/' {$channel=HIDDEN;}
    ;

