tree grammar CminusTW;

options {
    output=template;
    tokenVocab=Cminus;
    ASTLabelType=CommonTree;
}

@header {
    import java.lang.Double;
    import java.util.HashMap;
}

@members {
    HashMap locals = new HashMap();
    int localsLimit = 4;
    int labelCount = 1;

    public String genLabel() {
        return "LL" + (labelCount++);
    }
}

program
    : statements -> program(stackLimit={10}, localsLimit={localsLimit+1}, statements={$statements.st})
    ;

statements
    : /* empty */ -> empty()
    | s1=statement s2=statements 
      -> concat(i1={$s1.st}, i2={$s2.st})
    ;

statement
    : declaration
    | assignment
      -> {$assignment.st}
    | print
      -> {$print.st}
    | input
      -> {$input.st}
    | ifnt
      -> {$ifnt.st}
    | whilent
      -> {$whilent.st}
    ;
    
declaration
    : ^(VARDEF ID)
      {
        if (!locals.containsKey($ID.text)) {
            locals.put($ID.text, localsLimit);
            localsLimit += 2;
        }
      }
    ;

assignment
    : ^(ASSIGN ID expr)
      -> store(id={$ID.text}, n={locals.get($ID.text)}, v={$expr.st})
    ;

print
    : ^(PRINT expr)
      -> print(v={$expr.st})
    ;

input
    : ^(INPUT ID)
      -> input(id={$ID.text}, n={locals.get($ID.text)})
    ;

ifnt
    : ^(IF condition b1=block b2=block)
      -> if(condition={$condition.st}, thenBlock={$b1.st}, elseBlock={$b2.st}, label1={genLabel()}, label2={genLabel()})
    ;

whilent
    : ^(WHILE condition b1=block)
      -> while(condition={$condition.st}, block={$b1.st}, label1={genLabel()}, label2={genLabel()})
    ;

condition
    : ^(LT e1=expr e2=expr)
      -> lt(e1={$e1.st}, e2={$e2.st}, label1={genLabel()}, label2={genLabel()})
    | ^(GT e1=expr e2=expr)
      -> gt(e1={$e1.st}, e2={$e2.st}, label1={genLabel()}, label2={genLabel()})
    | ^(EQ e1=expr e2=expr)
      -> eq(e1={$e1.st}, e2={$e2.st}, label1={genLabel()}, label2={genLabel()})
    ;

block
    : ^(BLOCK statements)
      -> {$statements.st}
    ;

expr
    : ^(UPLUS e1=expr)
      -> uplus(e1={$e1.st})        
    | ^(UMINUS e1=expr)
      -> uminus(e1={$e1.st}) 
    | ^(PLUS e1=expr e2=expr)
      -> add(e1={$e1.st}, e2={$e2.st})
    | ^(MINUS e1=expr e2=expr)
      -> sub(e1={$e1.st}, e2={$e2.st})
    | ^(MULT e1=expr e2=expr)
      -> mul(e1={$e1.st}, e2={$e2.st})
    | ^(DIV e1=expr e2=expr)
      -> div(e1={$e1.st}, e2={$e2.st})
    | ^(POW e1=expr e2=expr)
      -> pow(e1={$e1.st}, e2={$e2.st})
    | DOUBLE
      -> push(const={Double.parseDouble($DOUBLE.text)})
    | PI
      -> push(const={"3.14159265358979323846"})
    | E
      -> push(const={"2.71828182845904523536"})
    | ID
      -> load(id={$ID.text}, n={locals.get($ID.text)})
    ;
