import org.antlr.runtime.*;
import org.antlr.runtime.tree.*;
import org.antlr.stringtemplate.*;
import java.io.*;

public class Compiler {
    public static void main(String[] args) throws Exception {
        // load template
        FileReader groupFileR = new FileReader("CminusTW.stg");
        StringTemplateGroup templates = new StringTemplateGroup(groupFileR);
        groupFileR.close();

        // create a CharStream that reads from standard input
        ANTLRInputStream input = new ANTLRInputStream(System.in);
        // create a lexer that feeds off of input CharStream
        CminusLexer lexer = new CminusLexer(input);
        // create a buffer of tokens pulled from the lexer
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        // create a parser that feeds off the tokens buffer
        CminusParser parser = new CminusParser(tokens);
        // begin parsing at rule r
        CminusParser.program_return r = parser.program();

        CommonTree t = (CommonTree)r.getTree();
        System.out.println("; " + t.toStringTree());


        // Walk resulting tree; create treenode stream first
        CommonTreeNodeStream nodes = new CommonTreeNodeStream(t);
        // AST nodes have payloads that point into token stream
        nodes.setTokenStream(tokens);
        // Create a tree Walker attached to the nodes stream
        CminusTW walker = new CminusTW(nodes);
        // use loaded template while walking the tree
        walker.setTemplateLib(templates);
        // Invoke the start symbol, rule program
        CminusTW.program_return r2 = walker.program();

        StringTemplate output = (StringTemplate)r2.getTemplate();
        System.out.println(output.toString());
    }
}
